@@ -1,12 +1,12 @@
 #ifndef BASICFRAME_H		// -*- c++ -*-
 #define BASICFRAME_H
 ///
-// Copyright (C) 2002, 2003, Fredrik Arnerup & Rasmus Kaj, See COPYING
+// Copyright (C) 2002 - 2004, Fredrik Arnerup & Rasmus Kaj, See COPYING
 ///
 #include "pagent.h"
 #include "fileerrors.h"
+#include "util/xmlwrap.h"
 #include <iostream>
-#include <libxml++/libxml++.h>
 
 namespace Error {
   struct FrameType: public File {
@@ -16,15 +16,19 @@ namespace Error {
 
 class BasicFrame: public Pagent {
 public:
-  BasicFrame(Group *parent, float w, float h, const std::string& name);
-  BasicFrame(Group *parent, const xmlpp::Element& node);
+  BasicFrame(Group *parent, const std::string& name);
+  BasicFrame(const ElementWrap& xml, Group *parent);
   virtual ~BasicFrame();
   
   virtual xmlpp::Element *save(xmlpp::Element& parent_node,
 			       const FileContext &context) const;
   virtual void print(std::ostream &out, bool grayscale = false) const;
-  
-  Boundary get_box() const;
+
+  /**
+   * This implementation of get_obstacle_boundary() returns the bounding box
+   * for the frame, appropriatley grown.  Pagents that works as
+   * non-rectangular boundaries should provide their own implementation.
+   */
   Boundary get_obstacle_boundary() const;
   
   // Errors reported by generate_picture:
