@@ -28,15 +28,6 @@ SigC::Signal1<void, DocRef> Document::selection_changed_signal;
 SigC::Signal1<void, DocRef> Document::streams_changed_signal;
 
 namespace {			// local
-  class same_name {
-  public:
-    same_name(const std::string& name) : name_(name) {};
-    bool operator() (const TextStream* stream) {
-      return stream->get_name() == name_;
-    }
-  private:
-    const std::string name_;
-  };
   class NameOrder {
   public:
     bool operator() (const TextStream* x, const TextStream* y) const {
@@ -88,26 +79,26 @@ Document::Document(const std::string &template_file_)
 Document::~Document() {
   debug << "destroying Document" << std::endl;
 
+  // delete the streams before the pages, or the textframes will try
+  // to run the typesetter threads from their destructors
+  while(!text_streams.empty()) {
+    TextStream *text_stream = *text_streams.begin();
+    text_streams.pop_front();
+    delete text_stream;
+  }
+
   // delete pages
   while(!pages.empty()) {
     Page *page = *pages.begin();
     pages.pop_front();
     delete page;
   }
-
-  // delete streams
-  while(!text_streams.empty()) {
-    TextStream *text_stream = *text_streams.begin();
-    text_streams.pop_front();
-    delete text_stream;
-  }
 }
 
 std::string Document::make_up_new_name() {
   while(stream_num < INT_MAX) {
     std::string tmp = "stream" + tostr(stream_num);
-    if(find_if(text_streams.begin(), text_streams.end(), same_name(tmp))
-       == text_streams.end())
+    if(!get_text_stream(tmp))
       return tmp;
     // Up the number only afterwards if no match, so if the user creates a
     // stream, renames it, and creates another one, he doesn't lose numbers.
@@ -192,11 +183,15 @@ void Document::rename_text_stream(const std::string &old_name,
 				  const std::string &new_name) {
   TextStream *stream = get_text_stream(old_name);
   if(!stream)
-    throw Error::TextStreamName();
+    throw Error::TextStreamName("A stream with the name \"" + old_name
+                                + "\" does not exist.");
   if(old_name == new_name)
     return; //not an error
+  if(new_name.empty())
+    throw Error::TextStreamName("The stream must have a name");
   if(get_text_stream(new_name))
-    throw Error::TextStreamName();
+    throw Error::TextStreamName("A stream with the name \"" + new_name
+                                + "\" already exists");
   stream->set_name(new_name);
   streams_changed_signal(self());
 }
@@ -207,33 +202,38 @@ void Document::add_text_stream(TextStream* new_stream) {
 }
 
 void Document::_add_text_stream(TextStream* new_stream) {
-  for(StreamVec::const_iterator i = text_streams.begin();
-      i != text_streams.end(); i++)
-    if((*i)->get_name() == new_stream->get_name())
-      throw Error::TextStreamName();
-
+  const std::string& name = new_stream->get_name();
+  if(name.empty())
+    throw Error::TextStreamName("The stream must have a name");
+  if(get_text_stream(name))
+    throw Error::TextStreamName("A stream with the name \"" + name
+                                + "\" already exists");
   text_streams.push_back(new_stream);
 }
 
+namespace {
+  template<class A>
+  struct NameIs {
+    std::string _name;
+    NameIs(const std::string &name) : _name(name) {}
+    bool operator() (A *a) { return a->get_name() == _name; }
+  };
+}
+
 TextStream* Document::get_text_stream(const std::string &name) {
-  for(StreamVec::iterator i = text_streams.begin();
-      i != text_streams.end();
-      i++)
-    if((*i)->get_name() == name)
-      return *i;
-  return 0;
+  StreamVec::iterator i = std::find_if(text_streams.begin(), text_streams.end(),
+                                       NameIs<TextStream>(name));
+  return i != text_streams.end() ? *i : 0;
 }
 
 void Document::remove_text_stream(const std::string &name) {
-  for(StreamVec::iterator i = text_streams.begin();
-      i != text_streams.end();
-      i++)
-    if((*i)->get_name() == name) {
-      delete *i;
-      text_streams.erase(i);
-      streams_changed_signal(self());  
-      return;
-    }
+  StreamVec::iterator i = std::find_if(text_streams.begin(), text_streams.end(),
+                                       NameIs<TextStream>(name));
+  if(i != text_streams.end()) {
+    delete *i;
+    text_streams.erase(i);
+    streams_changed_signal(self());  
+  }
 }
 
 unsigned int Document::get_num_of_pages() const {
@@ -265,43 +265,39 @@ void Document::delete_page(int page_num) {
 }
 
 Page *Document::new_page(int page_num, Page *original) {
-  PageVec::iterator i = pages.begin();
-  int num_of_pages = get_num_of_pages();
-  Page *the_new_page;
-
+  std::auto_ptr<Page> the_new_page;
+  
   if(original) {
     xmlpp::Document tmpdoc;
-    xmlpp::Element *root = tmpdoc.create_root_node("template");
+    tmpdoc.create_root_node("template");
     FileContext context;
-    xmlpp::Element *tmp = original->save(*root, context);
-    the_new_page = new Page(*tmp, *this, context);
+    the_new_page.reset
+      (new Page(ElementWrap("no-file",
+                            *original->save(*tmpdoc.get_root_node(),  context)),
+                *this));
     the_new_page->set_name("");  // the name should not be inherited
-    /// \todo Check that Document desctructor destructs entire tree so we
-    /// don't leak memory here!
   } else {
-    the_new_page = new Page(*this);
+    the_new_page.reset(new Page(*this));
   }
-
+  
+  Page *result = the_new_page.get();
+  int num_of_pages = get_num_of_pages();
+  
   if(num_of_pages == 0) {
-    pages.push_front(the_new_page);
+    pages.push_front(the_new_page.release());
   } else if(page_num >= first_page_num 
 	    && page_num <= first_page_num + num_of_pages) {
     int j = first_page_num;
-
-    while(j < page_num && j < first_page_num + num_of_pages) {
-      i++;
-      j++;
-    }
-      
-    pages.insert(i, the_new_page);
+    PageVec::iterator i = pages.begin();
+    while(j < page_num) { i++; j++; }
+    
+    pages.insert(i, the_new_page.release());
   } else {
-    delete the_new_page;
-    the_new_page = 0;
     throw Error::InvalidPageNum(page_num);
   }
-
+  
   changed_signal(self());
-  return the_new_page;
+  return result;
 }
 
 int Document::get_page_num_of_page(const Page *page) const {
@@ -330,12 +326,9 @@ Page *Document::get_page(int page_num) {
 }
 
 Page *Document::get_page(const std::string &page_name) {
-  for(PageVec::iterator i = pages.begin();
-      i != pages.end(); i++)
-    if((*i)->get_name() == page_name)
-      return *i;
-
-  return 0;
+  PageVec::iterator i = std::find_if(pages.begin(), pages.end(),
+                                     NameIs<Page>(page_name));
+  return i != pages.end() ? *i : 0;
 }
 
 std::list<std::string> Document::get_template_pages() {
@@ -363,8 +356,12 @@ void Document::open(const std::string &filename) {
   try {
     xmlpp::DomParser tree(filename);
     /// \todo  Check if get_document is guaranteed to not return 0.
-    xml_open(tree.get_document()->get_root_node(), FileContext(filename));
-  } 
+    xmlpp::Element *rootnode = tree.get_document()->get_root_node();
+    if(rootnode)
+      xml_open(ElementWrap(filename, *rootnode));
+    else
+      throw std::runtime_error("No such file or no rootnode");
+  }
   catch(const xmlpp::exception& e) {
     throw OpenFailed(filename, e); /// \todo Signify the "bad xml" level?
   } 
@@ -389,13 +386,11 @@ DocRef Document::self() {
   return tmp;
 }
 
-void Document::xml_open(xmlpp::Element *root, const FileContext &context) {
+void Document::xml_open(const ElementWrap& xml) {
   /// \todo this function is too long and unreadable
-  std::string temp_template = "";
+  std::string temp_template = xml.get_filename("template");
 
-  if(!root)
-    throw Error::Read("Root node is NULL.");
-  if(root->get_name() != "document")
+  if(xml.get_element_name() != "document")
     throw Error::Read("Root node is not <document>");
 
   //default values if attribute is not encountered:
@@ -405,14 +400,13 @@ void Document::xml_open(xmlpp::Element *root, const FileContext &context) {
   paper_name = "A4";
 
   //read document attributes:
-  xmlpp::Element::AttributeList attributes = root->get_attributes();
+  xmlpp::Element::AttributeList attributes = xml.element().get_attributes();
   for(xmlpp::Element::AttributeList::iterator i = attributes.begin();
       i != attributes.end();
       i++)
     {
       const std::string name = (*i)->get_name();
-      if(name == "template")
-	temp_template = (*i)->get_value();
+      if(name == "template");	// handled elsewhere
       else if(name == "doublesided")
 	doublesided = to<bool>((*i)->get_value());
       else if(name == "landscape") {
@@ -433,18 +427,19 @@ void Document::xml_open(xmlpp::Element *root, const FileContext &context) {
     }
 
   if(!is_template) // templates can't have templates
-    set_template(context.from(temp_template)); 
+    set_template(temp_template);
   // the template overrides anything  explicitly stated in the document
 
   // read text streams and pages:
-  xmlpp::Element::NodeList children = root->get_children();
+  xmlpp::Element::NodeList children = xml.element().get_children();
   for(xmlpp::Node::NodeList::iterator i = children.begin();
       i != children.end();
       i++) {
     if(xmlpp::Element *elem = dynamic_cast<xmlpp::Element*>(*i)) {
       std::string name = elem->get_name();
       if(name == "text_stream") {
-	std::auto_ptr<TextStream> stream(new TextStream(*elem, context));
+	std::auto_ptr<TextStream> stream
+	  (new TextStream(ElementWrap(xml, *elem)));
 	const std::string &name = stream->get_name();
 	if(name.empty())
 	  throw Error::Read("Text stream has no name.");
@@ -456,9 +451,9 @@ void Document::xml_open(xmlpp::Element *root, const FileContext &context) {
 
 	
       } else if(name == "page") {
-	pages.push_back(new Page(*elem, *this, context));
+	pages.push_back(new Page(ElementWrap(xml, *elem), *this));
       } else
-	warning << "Unknown node <" << elem->get_name()
+	warning << "Unknown node <" << name
 		<< "> ignored in <document>" << std::endl;
     }
   }
@@ -645,10 +640,8 @@ void Document::print(std::ostream& out, int first_page, int last_page,
       }
       out << "%%BeginResource: font " << fm->unalias(*i) << std::endl;
       if(suffix(fontfile) == "pfa") { // ascii
-	std::string tmpline;
-	while(getline(in, tmpline)) {
-	  out << tmpline << std::endl;
-	}
+	out << in.rdbuf();
+	
       } else { // assume it is pfb
 	try {
 	  PS::pfb2pfa(in, out);
