@@ -18,10 +18,6 @@
 #include "document/document.h"
 #include "document/page.h"
 
-#include "icons/wait.xpm"
-#include "icons/missing.xpm"
-#include "icons/broken.xpm"
-
 #include "util/warning.h"
 #include "util/stringutil.h"
 #include "util/filesys.h"
@@ -34,10 +30,6 @@
 #include "config.h"
 #include "lengthunits.h"
 
-IVector DocumentView::get_origin() const {
-  return IVector(margin_size, margin_size);
-}
-
 float DocumentView::pt2scr(float pt) const {
    return int(resolution * zoom_factor 
 	      * length_units.from_base(pt, "in") + 0.5);
@@ -48,24 +40,20 @@ float DocumentView::scr2pt(float scr) const {
 }  
 
 Gdk::Point DocumentView::pt2scr(const Vector& pt) const {
-  return Gdk::Point(int(get_origin().x + pt2scr(pt.x) + 0.5),
-		    int(get_origin().y + page_height() - pt2scr(pt.y) + 0.5));
+  double x, y;
+  world_to_window(pt.x, -pt.y, x, y);
+  return Gdk::Point(int(x + 0.5), int(y + 0.5));
 }
 
 Vector DocumentView::scr2pt(const Gdk::Point& scr) const {
-  return Vector(scr2pt(scr.get_x() - get_origin().x),
-		scr2pt(page_height() - (scr.get_y() - get_origin().y)));
-}
-
-int DocumentView::page_height() const {
-  return int(document ? pt2scr(document->get_height()) + 0.5 : 0);
-}
-
-int DocumentView::page_width() const {
-  return int(document ? pt2scr(document->get_width()) + 0.5 : 0);
+  Vector w;
+  window_to_world(scr.get_x(), scr.get_y(), w.x, w.y);
+  w.y = -w.y;
+  return w;
 }
 
 
+//////////////////////////////////// BEGIN JUNK
 void DocumentView::insert_page_before() {
   if(document)
     template_page_dialog->show_it(true, 
@@ -83,15 +71,82 @@ void DocumentView::insert_page_after() {
 }
 
 void DocumentView::delete_page() {
-  try {
-    if(document)
-      document->delete_page(current_page_num);
-  } 
-  catch(const Error::InvalidPageNum &e) {
-    warning << "DocumentView::delete_page: " << e.what() << std::endl;
+  if(document) {
+    pageview.clear(); // Do this first, so the viewent won't be
+                      // listening to the selection_changed_signal
+                      // which is emitted after the page has been
+                      // deleted.
+    document->delete_page(current_page_num);
   }
 }
 
+
+void DocumentView::new_text_frame(TextStream *stream) {
+  Page *page = get_page();
+  if(!page || !document)
+    return;
+  
+  int w = 200, h = 300;
+  // default to the first (alphabetically) stream
+  page->addObject(new TextFrame(page, stream, w, h));
+}
+
+void DocumentView::new_image_frame(std::string filename, float res) {
+  Page *page = get_page();
+  if(!page)
+    return;
+  get_document()->select_all(false);
+  
+  /// \todo more helpful error messages
+  if(ImageFrame::is_postscript(filename))
+    page->addObject(new ImageFrame(page, filename));
+  else {
+    try {
+      page->addObject(new RasterFrame(page, filename, res));
+    } catch(const std::exception& e){
+      throw UserError("Could not open \"" + filename
+		      + "\"", e);
+    }
+  }
+}
+
+void DocumentView::select_all_frames() {
+  if(Page *page = get_page()) {
+      page->select_all(true); 
+    }
+}
+
+void DocumentView::unselect_all_frames() {
+  if(Page *page = get_page()) {
+      page->select_all(false); 
+    }
+}
+
+void DocumentView::delete_selected() {
+  if(DocRef doc = get_document()) {
+    doc->delete_selected();
+  }
+}  
+
+void DocumentView::group_selected() {
+  if(Page *page = get_page()) {
+    page->group_selected();
+  }
+}
+
+void DocumentView::ungroup_selected() {
+  if(Page *page = get_page()) {
+    page->ungroup_selected();
+  }
+}
+
+void DocumentView::rearrange_selected(RearrangeTarget target) {
+  if(Page *page = get_page()) {
+    page->rearrange_selected(target);
+  }
+}  
+//////////////////////////////////////END JUNK
+
 Page *DocumentView::get_page() {
   if(document) {
     Page *page = document->get_page(current_page_num);
@@ -117,10 +172,15 @@ void DocumentView::act_on_document_change(DocRef document_) {
     current_page_num = last;
     current_page_num_changed_signal();
   }
-  draw();
   document_changed_signal();
 }
 
+void DocumentView::act_on_selection_change(DocRef document_) {
+  if(!document || document != document_)
+    return;
+  update_handles();
+}
+
 void DocumentView::set_current_page_num(int current) {
   if(!document)
     return;
@@ -134,36 +194,65 @@ void DocumentView::set_current_page_num(int current) {
   if(current != current_page_num) {
       current_page_num = current;
       current_page_num_changed_signal();
-      draw(); 
     }
+  if(Page *page = get_page()) {
+    pageview = core.getMeta("page")->create_viewent(*this, *page);
+  }
+}
+
+void DocumentView::update_document_size(DocRef document_) {
+  if(!document || document != document_)
+    return;
+  float w = document->get_width();
+  float h = document->get_height();
+  
+  reset_scroll_region();
+  
+  no_pages_frame->property_x1() = 0;
+  no_pages_frame->property_y1() = 0;
+  no_pages_frame->property_x2() = w;
+  no_pages_frame->property_y2() = -h;
+  
+  no_pages_text->property_x() = w / 2;
+  no_pages_text->property_y() = -h / 1.8;
 }
 
 void DocumentView::set_document(DocMeta d) {
   if(document == d)
     return;
 
-  pageview.clear();
-
   document = d;
+  update_handles();
   if(document) {
+    update_document_size(document.get_document());
+    no_pages_group.show();
+    
     current_page_num = document->get_first_page_num();
+    
+    Document::size_changed_signal.connect
+      (slot(*this, &DocumentView::update_document_size));
     Document::changed_signal.connect
       (slot(*this, &DocumentView::act_on_document_change));
-    Document::size_changed_signal.connect
-      (slot(*this, &DocumentView::act_on_size_change));
+    Document::selection_changed_signal.connect
+      (slot(*this, &DocumentView::act_on_selection_change));
+  } else {
+    no_pages_group.hide();
   }
 
   document_changed_signal(); //?
   document_set_signal();
 
-  adjust_size();
-  draw();
+  if(Page *page = get_page()) {
+    pageview = core.getMeta("page")->create_viewent(*this, *page);
+  } else {
+    pageview.clear();
+  }
 }
 
 DocumentView::DocumentView(DocMeta d)
-  : snap_mode(snap::NONE), document(d),
-    old_page(0),
-    wait_pixmap(0), missing_pixmap(0), broken_pixmap(0)
+  : no_pages_group(*root()), pagent_group(*root()), guide_group(*root()),
+    handle_group(*root()),
+    snap_mode(snap::NONE), document(DocRef()), old_page(0)
 {
   set_events(Gdk::EXPOSURE_MASK
 	     | Gdk::LEAVE_NOTIFY_MASK
@@ -189,11 +278,27 @@ DocumentView::DocumentView(DocMeta d)
   resolution = (xresolution + yresolution) / 2;  // average :-)
   verbose << "Screen resolution: " << resolution << " dpi" << std::endl;
 
+  set_pixels_per_unit(resolution / 72.0 * zoom_factor);
+  
+  using namespace Gnome::Canvas;
+  no_pages_frame.reset(new Rect(no_pages_group));
+  no_pages_frame->property_outline_color_gdk() = get_color(Color::frame);
+  no_pages_frame->property_width_pixels() = 1;
+  no_pages_text.reset(new Text(no_pages_group, 0, 0, "No pages"));
+  no_pages_text->property_anchor() = Gtk::ANCHOR_CENTER;
+  no_pages_group.hide();
+  
+  for(int i = 0; i < 8; i++) {
+    handles[i] = manage(new Rect(handle_group));
+    handles[i]->property_fill_color_gdk() = get_color(Color::bg);
+    handles[i]->property_outline_color_gdk() = get_color(Color::frame);
+    handles[i]->property_width_pixels() = 1;
+  }
+
   set_document(d);
+
   reshaping = moving = false;
   margin_size = 30;
-  Page::ready_to_draw_page_signal.connect
-    (slot(*this, &DocumentView::maybe_page_ready_to_draw));
 
   // drag'n drop stuff:
   std::list<Gtk::TargetEntry> targets;
@@ -205,39 +310,21 @@ DocumentView::DocumentView(DocMeta d)
   targets.push_back(Gtk::TargetEntry("UTF8_STRING"));
   targets.push_back(Gtk::TargetEntry("STRING"));
   drag_dest_set(targets);
-}
 
-void DocumentView::maybe_page_ready_to_draw(Pagent *pagent) {
-  if(pagent == get_page())
-    draw(moving || reshaping);
+  // Update handles when we turn pages
+  current_page_num_changed_signal.connect
+    (slot(*this, &DocumentView::update_handles));
 }
 
 DocumentView::~DocumentView() {
 }
 
-void DocumentView::adjust_size() {
-  if(document)
-    set_size_request(2 * margin_size 
-		     + int(pt2scr(document->get_width()) + 0.5), 
-		     2 * margin_size
-		     + int(pt2scr(document->get_height()) + 0.5));
-}
-
-void DocumentView::act_on_size_change(DocRef document_) {
-  if(document && document == document_) {
-    hide();
-    adjust_size();
-    show();
-  }
-}
-
 void DocumentView::set_zoom_factor(float factor) {
   if(factor > 0 && document) {
       zoom_factor = factor;
-      hide();
-      adjust_size();
+      set_pixels_per_unit(resolution / 72.0 * zoom_factor);
       zoom_change_signal(zoom_factor);
-      show();
+      update_handles();
     }
 }
 
@@ -248,9 +335,8 @@ bool DocumentView::in_move_area(int x, int y) {
     const Vector pos = scr2pt(Gdk::Point(x, y));
     const float dist = scr2pt(2);
 
-    for(Document::Selection::const_iterator i=selected.begin(); 
- 	i != selected.end(); 
- 	++i) 
+    for(Document::Selection::const_iterator 
+          i = selected.begin(); i != selected.end(); ++i) 
       if(&Page::containing(**i) == page 
 	 && (*i)->get_box()->isInsideOrClose(pos, dist))
  	return true;
@@ -263,9 +349,8 @@ Pagent* DocumentView::in_select_area(int x, int y) {
     const Vector pos = scr2pt(Gdk::Point(x, y));
     const float dist = scr2pt(2);
     
-    for(Group::ChildVec::const_iterator i = page->pbegin(); 
-	i != page->pend(); 
-	i++) {
+    for(Group::ChildVec::const_iterator
+          i = page->pbegin(); i != page->pend(); i++) {
       if((*i)->get_box()->isInsideOrClose(pos, dist))
 	return *i;
     }
@@ -289,34 +374,15 @@ namespace {
   }
 }
 
-namespace {
-  // finds out if there is a resize handle at (x, y) and puts its number in j
-  bool hover_reshape(const DocumentView& view, int x, int y, int& j) {
-    if(!view.get_document())
-      return false;
-
-    const Document::Selection selected = view.get_document()->selected();
-    if(selected.size() != 1) 
-      return false;
-    const Pagent& obj = *selected.front();
-    
-    // "paper coordinates"
-    const int dist = int(config.ReshapeBoxSize.values.front()) / 2;
-
-    ResizeHandles handles = get_resize_handles(obj);
-    j = 0;
-    for(ResizeHandles::const_iterator i = handles.begin();
-	i != handles.end(); i++) {
-      const Gdk::Point pos = view.pt2scr(*i);
-      if(chessboard_dist(Vector(pos.get_x(), pos.get_y()), Vector(x, y)) 
-	 <= dist) {
-	return true;
-      }
-      j++;
-    }
-
-    return false;
-  }
+int DocumentView::hover_reshape(int x, int y) {
+  if(!document)
+    return -1;
+  Vector mouse = scr2pt(Gdk::Point(x, y));
+  Gnome::Canvas::Item *item = get_item_at(mouse.x, -mouse.y);
+  for(int i = 0; i < 8; i++)
+    if(item == handles[i])
+      return i;
+  return -1;
 }
 
 bool DocumentView::on_button_press_event(GdkEventButton *event) {
@@ -331,9 +397,8 @@ bool DocumentView::on_button_press_event(GdkEventButton *event) {
   if(page && event->button == 1) {
     const Document::Selection selected = get_document()->selected();
 
-    int j;
-    bool on_handle = hover_reshape(*this, int(event->x), int(event->y), j);
-    if(on_handle && selected.size() == 1
+    int j = hover_reshape(int(event->x), int(event->y));
+    if(j != -1 && selected.size() == 1
        && !selected.front()->get_lock()) { // maybe start moving/resizing?
       // reshape
 
@@ -355,59 +420,11 @@ bool DocumentView::on_button_press_event(GdkEventButton *event) {
     } else if((event->state & Gdk::CONTROL_MASK) == 0) { // deselect all
       get_document()->select_all(false);
     }
-    
-    draw();
   }
 
   return true;
 }
 
-void DocumentView::new_text_frame(TextStream *stream) {
-  Page *page = get_page();
-  if(!page || !document)
-    return;
-  
-  int w = 200, h = 300;
-  // default to the first (alphabetically) stream
-  page->addObject(new TextFrame(page, stream, w, h));
-
-  draw();
-}
-
-void DocumentView::new_image_frame(std::string filename, float res) {
-  Page *page = get_page();
-  if(!page)
-    return;
-  get_document()->select_all(false);
-  
-  /// \todo more helpful error messages
-  if(ImageFrame::is_postscript(filename))
-    page->addObject(new ImageFrame(page, filename, 72, 72));
-  else {
-    try {
-      page->addObject(new RasterFrame(page, filename, res));
-    } catch(const std::exception& e){
-      throw UserError("Could not open \"" + filename
-		      + "\"", e);
-    }
-  }
-  draw();
-}
-
-void DocumentView::select_all_frames() {
-  if(Page *page = get_page()) {
-      page->select_all(true); 
-      draw();
-    }
-}
-
-void DocumentView::unselect_all_frames() {
-  if(Page *page = get_page()) {
-      page->select_all(false); 
-      draw();
-    }
-}
-
 // *** copy/paste stuff ***/
 
 namespace {
@@ -508,66 +525,43 @@ DocumentView::on_clipboard_received(const Gtk::SelectionData& selection_data)
 
     get_document()->select_all(false);
     for(xmlpp::Element::NodeList::const_iterator i = children.begin();
-	i != children.end();
-	i++)
+	i != children.end(); i++)
       {
 	if(xmlpp::Element *elem = dynamic_cast<xmlpp::Element*>(*i)) {
 	  if(elem->get_name() == "text_stream") {
-	    std::auto_ptr<TextStream> stream(new TextStream(*elem, 
-							    FileContext()));
+	    std::auto_ptr<TextStream> stream(new TextStream
+					     (ElementWrap("", *elem)));
 	    // Don't try to add a stream with a name that already exists
 	    /// \todo rename the stream automatically?
 	    if(!document->get_text_stream(stream->get_name()))
 	      document->add_text_stream(stream.release());
 	  } else {
-	    Pagent *pagent = load(*elem, page, FileContext());
+	    Pagent *pagent = load(ElementWrap("", *elem), page);
 	    page->add(pagent);
 	    get_document()->select(pagent, false); // don't deselect
 	  }
 	}
       }
-    draw();
   } catch(const std::exception& e) {
     throw std::runtime_error("Error encountered when pasting:\n" 
 			     + std::string(e.what()));
   }
 }  
 
-void DocumentView::delete_selected() {
-  if(DocRef doc = get_document()) {
-    doc->delete_selected();
-    draw();
-  }
-}  
-
-void DocumentView::group_selected() {
-  if(Page *page = get_page()) {
-    page->group_selected();
-    draw();
-  }
-}
-
-void DocumentView::ungroup_selected() {
-  if(Page *page = get_page()) {
-    page->ungroup_selected();
-    draw();
-  }
-}
-
-void DocumentView::rearrange_selected(RearrangeTarget target) {
-  if(Page *page = get_page()) {
-    page->rearrange_selected(target);
-    draw();
-  }
-}  
-
+// Todo: This method should be removed.  Instead, there should exist something
+// (not related to a specific view, but to the document), that listens to
+// geometry changes, finds all text streams that have frames that intersects
+// with the object (before or after the change) and retypesets them.
+// What is done here is to much typsetting when objects moved / resized by the
+// view, and no typesetting at all when objects are moved / resized by e.g.
+// the properties dialog.
 void DocumentView::refresh_streams() {
   if(Page *page = get_page()) {
     Document::StreamVec streams;
     for(Group::ChildVec::const_iterator i = page->pbegin();
 	i != page->pend();
-	i++)
-      {	
+	i++) 
+      {
 	TextFrame *tmp = dynamic_cast<TextFrame*>(*i);
 	if(tmp) {
 	  TextStream *stream = tmp->get_stream();
@@ -580,7 +574,11 @@ void DocumentView::refresh_streams() {
 	i != streams.end();
 	i++)
       {
-	(*i)->run_typesetter();
+        try {
+          (*i)->run_typesetter();
+        } catch(const BasicFrame::GenPicError &e) {
+          warning << "Failed to refresh stream: " << e.what() << std::endl;
+        }
 	debug << (*i)->get_name() << ", " 
 	      << (*i)->get_association() << std::endl;
       }
@@ -604,7 +602,8 @@ void DocumentView::begin_reshape(int x, int y, int box) {
   // pointer is probably not in center of handle
   offset = IVector(x - handle.get_x(), 
 		   y - handle.get_y());
-  old_width = obj.get_width(); old_height = obj.get_height();
+
+  old_size = obj.get_inherent_size();
   old_matrix = obj.get_matrix();
   reshape_box = box; reshaping = true;
 }
@@ -617,12 +616,11 @@ void DocumentView::end_reshape(bool revert) {
       if(selected.size() != 1) {
 	Pagent& obj = *selected.front();
 	obj.set_matrix(old_matrix);
-	if(obj.is_resizable()) {
-	  obj.set_size(old_width, old_height);
+	if(Pagent::Resizable* o = dynamic_cast<Pagent::Resizable*>(&obj)) {
+	  o->set_size(old_size.x, old_size.y);
 	}
       }
     }
-    draw(false);
     refresh_streams();
   }
 }
@@ -663,7 +661,6 @@ void DocumentView::end_move(bool revert) {
   if(get_page()) {
     moving = false;
     refresh_streams();
-    draw(false);
   }
 }
 
@@ -677,8 +674,7 @@ bool DocumentView::on_key_press_event(GdkEventKey* key) {
   return false;
 }
 
-bool DocumentView::on_button_release_event(GdkEventButton *event)
-{
+bool DocumentView::on_button_release_event(GdkEventButton *event) {
   if(moving)
     end_move(false);
   if(reshaping)
@@ -702,7 +698,7 @@ Gdk::Cursor DocumentView::get_cursor(int x, int y) {
     return Gdk::TOP_LEFT_ARROW;
   
   int j = reshape_box;
-  if(reshaping || hover_reshape(*this, x, y, j))
+  if(reshaping || ((j = hover_reshape(x, y)) != -1))
     switch(j) {
     case 0: return Gdk::BOTTOM_LEFT_CORNER;
     case 1: return Gdk::LEFT_SIDE;
@@ -762,10 +758,7 @@ void DocumentView::move_reshape_box(int x, int y) {
 
   // get pos in matrix coordinates
   Vector pos = m.inv().transform(p);
-
-  float w = obj.get_width(), h = obj.get_height();
-
-  Vector ll(0, 0), ur(w, h); // lower left / upper right
+  Vector ll(0, 0), ur(obj.get_inherent_size()); // lower left / upper right
   switch(reshape_box) {
   case 0: // bottom left
     ll = pos;
@@ -834,11 +827,12 @@ void DocumentView::move_reshape_box(int x, int y) {
     return;
   }
 
-  if(obj.is_resizable()) {
-    obj.set_size(fabs(size.x), fabs(size.y));
+  if(Pagent::Resizable* o = dynamic_cast<Pagent::Resizable*>(&obj)) {
+    o->set_size(fabs(size.x), fabs(size.y));
   } else {
-    obj.set_scaling(m.sc_x() * size.x / obj.get_width(), 
-		    m.sc_y() * size.y / obj.get_height());
+    const Vector osize = obj.get_inherent_size();
+    obj.set_scaling(m.sc_x() * size.x / osize.x,
+		    m.sc_y() * size.y / osize.y);
   }
   obj.set_matrix(Matrix::translation(_ll) * obj.get_matrix());
 }
@@ -847,18 +841,16 @@ bool DocumentView::on_motion_notify_event(GdkEventMotion *event) {
   if(!get_page())
     return true;
 
-  // find out where the pointer is
-  int x, y;
-  Gdk::ModifierType state;
-  Glib::RefPtr<Gdk::Window> window = win;
-  if(event->is_hint)
-    window->get_pointer(x, y, state);
-  else {
-    x = int(event->x);
-    y = int(event->y);
-    state = Gdk::ModifierType(event->state);
+  // some silliness we have to do or we won't get new motion notify events
+  if(event->is_hint) {
+    Gdk::ModifierType state;
+    int x, y;
+    win->get_pointer(x, y, state);
   }
 
+  int x = int(event->x);
+  int y = int(event->y);
+  
   update_cursor(x, y);
 
   Document::Selection selected = get_document()->selected();
@@ -902,14 +894,25 @@ bool DocumentView::on_motion_notify_event(GdkEventMotion *event) {
 	  move_reshape_box(x, y);
       }
     }
-    draw(true); // reshaping
   }
 
   return true;
 }
 
+void DocumentView::reset_scroll_region() {
+  if(document) {
+    float w = document->get_width();
+    float h = document->get_height();
+    set_scroll_region(-margin_size, -h - margin_size, 
+                      w + margin_size, margin_size);
+  }
+}
+
 void DocumentView::on_realize() {
-  Gtk::DrawingArea::on_realize();
+  Gnome::Canvas::Canvas::on_realize();
+
+  // Doing this in the constructor does not seem to work all the time
+  reset_scroll_region();
 
   // get_toplevel doesn't work in the constructor
   Gtk::Window *toplevel = dynamic_cast<Gtk::Window*>(get_toplevel());
@@ -918,154 +921,6 @@ void DocumentView::on_realize() {
   template_page_dialog.reset(new TemplatePageDialog(*toplevel, *this));
 
   win = get_window();
-  gc = Gdk::GC::create(win);
-  Glib::RefPtr<Gdk::Bitmap> bitmap;
-  wait_pixmap = Gdk::Pixmap::create_from_xpm
-    ((const Glib::RefPtr<const Gdk::Drawable>&)win, bitmap, white, wait_xpm); 
-  missing_pixmap =
-    Gdk::Pixmap::create_from_xpm
-    ((const Glib::RefPtr<const Gdk::Drawable>&) win,
-     bitmap, white, missing_xpm); 
-  broken_pixmap =
-    Gdk::Pixmap::create_from_xpm
-    ((const Glib::RefPtr<const Gdk::Drawable>&) win,
-     bitmap, white, broken_xpm); 
-}
-
-void DocumentView::draw(bool reshaping) {
-  if(!is_realized())
-    return; // will get lots of Gdk-CRITICALs otherwise
-
-  Glib::RefPtr<Gtk::Style> style = get_style();
-  IVector origin = get_origin();
-
-  if(Page *page = get_page()) {
-    // draw the actual page
-    if(!pageview)
-      pageview = core.getMeta("page")->create_viewent(*this, *page);
-    pageview->draw_content();
-    
-//     // draw grid
-//     if(snap_mode == snap::Grid) {
-//       gc->set_line_attributes(1, Gdk::LINE_SOLID, Gdk::CAP_BUTT, 
-// 			      Gdk::JOIN_MITER);
-//       gc->set_foreground(get_color(Color::guide));
-//       for(float x = grid_offset.x; x < page->get_width(); x += grid_size_x) {
-// 	Vector v0(x, 0), v1(x, page->get_height());
-// 	Gdk::Point p0 = pt2scr(v0), p1 = pt2scr(v1);
-// 	win->draw_line(gc, p0.get_x(), p0.get_y(), p1.get_x(), p1.get_y());
-//       }
-//       for(float y = grid_offset.y; y < page->get_height(); y += grid_size_y) {
-// 	Vector v0(0, y), v1(page->get_width(), y);
-// 	Gdk::Point p0 = pt2scr(v0), p1 = pt2scr(v1);
-// 	win->draw_line(gc, p0.get_x(), p0.get_y(), p1.get_x(), p1.get_y());
-//       }
-//       // draw diagonal (debug)
-//       gc->set_foreground(get_color(Color::frame));
-//       Vector ur, ll;
-//       box_of_selected(ll, ur, *get_document());
-//       ll = Vector(snap_to_grid(ll.x, grid_offset.x, grid_size_x),
-// 		  snap_to_grid(ll.y, grid_offset.y, grid_size_y));
-//       ur = Vector(snap_to_grid(ur.x, grid_offset.x, grid_size_x),
-// 		  snap_to_grid(ur.y, grid_offset.y, grid_size_y));
-//       Gdk::Point p0 = pt2scr(ll), p1 = pt2scr(ur);
-//       win->draw_line(gc, p0.get_x(), p0.get_y(), p1.get_x(), p1.get_y());
-//     }
-
-    // draw resize handles
-    gc->set_line_attributes(1, Gdk::LINE_SOLID, Gdk::CAP_BUTT, 
-			    Gdk::JOIN_MITER);
-    const Document::Selection &selection = document->selected();
-    if(selection.size() == 1) {
-      const Pagent &pagent = *selection.front();
-      if(!pagent.get_lock() && &Page::containing(pagent) == page) {
-	const int size = int(config.ReshapeBoxSize.values.front());
-	ResizeHandles handles = get_resize_handles(pagent);
-	for(ResizeHandles::const_iterator i = handles.begin(); 
-	    i != handles.end(); i++) {
-	  const Gdk::Point p = pt2scr(*i);
-	  gc->set_foreground(get_color(Color::frame));
-	  win->draw_rectangle(gc, false, 
-			      p.get_x() - size/2, 
-			      p.get_y() - size/2, 
-			      size - 1, size - 1);
-	  gc->set_foreground(get_color(Color::bg));
-	  win->draw_rectangle(gc, true, p.get_x() - size/2 + 1, 
-			      p.get_y() - size / 2 + 1, 
-			      size - 2, size - 2);
-	}
-      }
-    }
-
-    // probably not necessary to draw background when reshaping
-    //if(!reshaping) 
-    {
-      int w = page_width() + origin.x;
-      int h = page_height() + origin.y;
-      
-      // draw widget background
-      style->apply_default_background(win, true, Gtk::STATE_NORMAL,
-				      Gdk::Rectangle(0, 0, 
-						     get_width(), origin.y),
-				      0, 0, get_width(), get_height());    
-      style->apply_default_background(win, true, Gtk::STATE_NORMAL,
-				      Gdk::Rectangle(0, 0, 
-						     origin.x, get_height()),
-				      0, 0, get_width(), get_height());    
-      style->apply_default_background(win, true, Gtk::STATE_NORMAL,
-				      Gdk::Rectangle(0, h, 
-						     get_width(), 
-						     get_height() - h),
-				      0, 0, get_width(), get_height());    
-      style->apply_default_background(win, true, Gtk::STATE_NORMAL,
-				      Gdk::Rectangle(w, 0, 
-						     get_width() - w,
-						     get_height()),
-				      0, 0, get_width(), get_height());    
-    }
-  } else if(document){
-    // show that we have a document, albeit without pages
-    Glib::ustring message = "No Pages";
-    int x, y, w, h;
-
-    w = page_width(); 
-    h = page_height();
-
-    Gdk::Rectangle rectangle(0, 0, get_width(), get_height());
-
-    // draw widget background
-    style->apply_default_background(win, true, Gtk::STATE_NORMAL,
-				    rectangle,
-				    0, 0, get_width(), get_height());    
-
-    Glib::RefPtr<Pango::Context> context = get_pango_context();
-    Glib::RefPtr<Pango::Layout> layout = Pango::Layout::create(context);
-    layout->set_text(message);
-    layout->get_pixel_size(x, y);
-    x = (w - x) / 2 + origin.x; y = (h - y) / 2 + origin.y; 
-
-    // draw box indicating paper size
-    style->paint_box(win, Gtk::STATE_NORMAL, Gtk::SHADOW_ETCHED_IN,
-		     rectangle, *this, "Foobar?",
-		     origin.x, origin.y, 
-		     w - 1, h - 1);
-
-    // draw text
-    style->paint_layout(win, Gtk::STATE_NORMAL, true,
-			rectangle, *this, "Foobar?",
- 			x, y,
- 			layout);
-  } else // just paint backgorund
-    style->apply_default_background(win, true, Gtk::STATE_NORMAL,
-				    Gdk::Rectangle(0, 0, 
-						   get_width(), get_height()),
-				    0, 0, get_width(), get_height());    
-  show();
-}
-
-bool DocumentView::on_expose_event(GdkEventExpose *event) {
-  draw();
-  return false;
 }
 
 void DocumentView::on_drag_data_received
@@ -1124,6 +979,40 @@ void DocumentView::on_drag_data_received
   context->drag_finish(success, false, time);
 }
 
+void DocumentView::update_handles() {
+  Pagent *pagent;
+  // Show handles if there is one and only one selected object and it
+  // is on the current page.and it isn't locked
+  if(!document 
+     || document->selected().size() != 1 
+     || !get_page()->has_child(pagent = document->selected().front())
+     || pagent->get_lock()) {
+    handle_group.hide();
+    return;
+  }
+  handle_group.show();
+  // Make sure we connect only to the currently selected pagent
+  if(last_selected != pagent) {
+    last_selected = pagent;
+    selected_geometry_connection.disconnect();
+    selected_props_connection.disconnect();
+    selected_geometry_connection = pagent->geometry_changed_signal.connect
+      (SigC::hide<Pagent*>(slot(*this, &DocumentView::update_handles)));
+    // this is for noticing if the pagent is being locked/unlocked
+    selected_props_connection = pagent->props_changed_signal.connect
+      (SigC::hide<Pagent*>(slot(*this, &DocumentView::update_handles)));
+  }
+  ResizeHandles pos = get_resize_handles(*pagent);
+  assert(pos.size() == 8);
+  const float hs = scr2pt(int(config.ReshapeBoxSize.values.front()) / 2);
+  for(int i = 0; i < 8; i++) {
+    handles[i]->property_x1() = pos[i].x - hs; 
+    handles[i]->property_x2() = pos[i].x + hs; 
+    handles[i]->property_y1() = -(pos[i].y - hs); 
+    handles[i]->property_y2() = -(pos[i].y + hs); 
+  }
+}
+
 const Gdk::Color& DocumentView::get_color(Color::Id color) const {
   switch(color) {
   case Color::bg:     return white;
