@@ -8,7 +8,7 @@
 #include "document/group.h"
 #include "docmeta.h"
 #include <gdkmm.h>
-#include <gtkmm/drawingarea.h>
+#include <libgnomecanvasmm.h>
 #include <memory> // auto_ptr
 
 class TemplatePageDialog;
@@ -19,7 +19,7 @@ namespace snap {
   enum Mode {GRID, GUIDE, NONE};
 }
 
-class DocumentView: public Gtk::DrawingArea, public View {
+class DocumentView: public Gnome::Canvas::Canvas, public View {
 public:
   SigC::Signal0<void> current_page_num_changed_signal;
   SigC::Signal0<void> document_changed_signal;
@@ -36,7 +36,6 @@ public:
   DocMeta get_document_meta() const { return document; }
   void set_current_page_num(int);
   int get_current_page_num() const { return current_page_num; }
-  void draw(bool reshaping = false);
   Page* get_page();
   float get_zoom_factor() const { return zoom_factor; }
   void set_zoom_factor(float);
@@ -57,14 +56,15 @@ public:
   void new_image_frame(std::string, float res = -1);
   void select_all_frames();
   void unselect_all_frames();
-  Glib::RefPtr<Gdk::Drawable> get_win() { return win; }
-  Glib::RefPtr<Gdk::GC> get_gc() { return gc; }
+
+  Gnome::Canvas::Group& get_pagent_group() { return pagent_group; }
+  Gnome::Canvas::Group& get_guide_group() { return guide_group; }
+
   const Gdk::Color& get_color(Color::Id color) const;
 
   void set_snap(snap::Mode mode) { snap_mode = mode; }
   snap::Mode get_snap() const { return snap_mode; }
 		
-  IVector get_origin() const;
   double get_scrres() const { return resolution * zoom_factor; }
   float pt2scr(float pt) const;
   float scr2pt(float scr) const;
@@ -73,18 +73,6 @@ public:
   // right- and left-handed coordinate systems
   Gdk::Point pt2scr(const Vector& pt) const;
   Vector scr2pt(const Gdk::Point& scr) const;
-  int page_height() const;
-  int page_width() const;
-
-  Glib::RefPtr<const Gdk::Pixmap> get_wait_pixmap() const {
-    return wait_pixmap;
-  }
-  Glib::RefPtr<const Gdk::Pixmap> get_missing_pixmap() const {
-    return missing_pixmap;
-  }
-  Glib::RefPtr<const Gdk::Pixmap> get_broken_pixmap() const {
-    return broken_pixmap;
-  }
 
   bool on_key_press_event(GdkEventKey*); // called from Window
 
@@ -95,14 +83,24 @@ private:
   void operator = (const DocumentView&);
 
   Glib::RefPtr<Gdk::Window> win;
-  Glib::RefPtr<Gdk::GC> gc;
   Viewent::Ref pageview;
-  
+  Gnome::Canvas::Group no_pages_group, pagent_group, guide_group, handle_group;
+  std::auto_ptr<Gnome::Canvas::Rect> no_pages_frame;
+  std::auto_ptr<Gnome::Canvas::Text> no_pages_text;
+  Gnome::Canvas::Rect *handles[8]; /// managed by the canvas
+
+  /// a connection to the selected pagents geometry_changed_signal
+  SigC::Connection selected_geometry_connection;
+  /// a connection to the selected pagents props_changed_signal
+  SigC::Connection selected_props_connection;
+  /// keep track of which pagent the connection is to
+  Pagent *last_selected;
+
   snap::Mode snap_mode;
   int margin_size;
   float resolution; //ppi
   float zoom_factor;
-  float old_width, old_height;
+  Vector old_size;
   Matrix old_matrix;
   IVector offset;
   int reshape_box, current_page_num;
@@ -110,20 +108,25 @@ private:
   DocMeta document;
   Page *old_page; //last page visited
   std::auto_ptr<TemplatePageDialog> template_page_dialog;
-  Glib::RefPtr<Gdk::Pixmap> wait_pixmap, missing_pixmap, broken_pixmap;
 
   void on_clipboard_get(Gtk::SelectionData& selection_data, guint info);
   void on_clipboard_clear();
   void on_clipboard_received(const Gtk::SelectionData& selection_data);
 
+  /// If there is a reshape box at pixel coordinates (x, y) return the
+  /// index of the box, otherwise return -1.
+  int hover_reshape(int x, int y);
+  /// Set the scroll region of the canvas to fit the size of the document
+  void reset_scroll_region();
+
   void on_realize();
-  bool on_expose_event(GdkEventExpose*);
   bool on_button_press_event(GdkEventButton*);
   bool on_button_release_event(GdkEventButton*);
   bool on_motion_notify_event(GdkEventMotion*);
   void on_drag_data_received(const Glib::RefPtr<Gdk::DragContext>& context, 
 			     int x, int y, GtkSelectionData* selection_data, 
 			     guint info, guint time);
+  void update_handles();
 
   void refresh_streams();
   void begin_move(int x, int y);
@@ -133,13 +136,12 @@ private:
   void move_reshape_box(int x, int y);
 
   void act_on_document_change(DocRef document_);
+  void act_on_selection_change(DocRef document_);
+  void update_document_size(DocRef document_);
   Gdk::Cursor get_cursor(int x, int y);
   void update_cursor(int x, int y);
   bool in_move_area(int x, int y);
   Pagent* in_select_area(int x, int y);
-  void adjust_size();
-  void act_on_size_change(DocRef document_);
-  void maybe_page_ready_to_draw(Pagent *pagent);
 
   SigC::Connection connect_zoom_change(SigC::Slot1<void, float> slot) {
     return zoom_change_signal.connect(slot);
@@ -147,10 +149,3 @@ private:
 };
 
 #endif
-
-
-
-
-
-
-
