@@ -5,6 +5,7 @@
 ///
 #include "viewent.h"
 #include "document/cachedframe.h"
+#include <sys/types.h>		// pid_t
 
 class PostscriptViewent : public Viewent {
 protected:
@@ -23,23 +24,21 @@ public:
   ~PostscriptViewent();
   
   void set_image(const std::string& psfile);
-  bool draw_content();
-  void clear();
   
 protected:
   const Pagent* get_frame() const { return frame; }
-  
+  void set_state(State state);
+  void on_geometry_changed(Pagent*);
 private:
+  void regenerate();
   void process_stopped(pid_t _pid, bool exited_normally, int exit_code);
   
+  std::auto_ptr<Gnome::Canvas::Pixbuf> canvas_pixbuf;
   FrameRef frame;
   pid_t pid;
   std::string tmp_file;
   int scale_factor;
   PixbufRef pixbuf;
-  /** There was a problem with the content */
-  bool content_is_broken;
-  SigC::Connection procstopcon, propscon, objectcon, zoomcon;
 };
 
 #endif
